# Instaply Push Notification System

The repository contains a java reference implementation of the web hook, used by Instaply servers to send push 
notifications to SDK enabled apps.

The reference implementation is based on the SDK Push Notifications Specification.

## Technology stack

The proxy is based on the [Spring Boot](http://projects.spring.io/spring-boot/) framework, which makes it a self 
contained web application.

It requires a Java JDK 8 (tested with the Oracle one) and use JPA/Hibernate for database access. The
REST controllers are implemented using [Spring MVC](http://docs.spring.io/spring/docs/current/spring-framework-reference/html/mvc.html).

## Build from source

The project uses [Gradle](https://gradle.org/) for the build, version 2.4 is bundled in the package. The project is
built using the command:

    ./gradlew assemble
    
This should produce an output similar to:

    :compileJava
    :compileGroovy
    :processResources
    :classes
    :jar
    :findMainClass
    :startScripts
    :distTar
    :distZip
    :war
    :bootRepackage
    :assemble
    
    BUILD SUCCESSFUL
    
    Total time: 6.53 secs
    
The executable jar is created in ./build/libs/ipns-proxy-ref-impl.jar

## <a name="config"></a>Configuration

### Properties file

The application can be configured with the following properties file:

    # The port used by the embedded tomcat server, default is 8080
    #server.port=8080
    
    # The credentials for the Http basic authentication, default values are:
    #authentication.user=instaply
    #authentication.password=ChangeMe!
    
    # APNS
    # path to the apns certificate
    apns.certificate=/path/to/certificate.p12
    # certificate password
    apns.password=instaply
    # Sandbox or production app, set to true only for apps released to the iTunes Store
    apns.isProduction=false
    
    ## Datasource
    spring.datasource.url=jdbc:mysql://localhost/mydb
    spring.datasource.username=root
    spring.datasource.password=root
    spring.datasource.driver-class-name=com.mysql.jdbc.Driver

### Datasource

The devices tokens are retrieved from a MySQL database. It only require a unique table named `customer_device`.

| Column        | Type         |
|---------------|--------------|
| id            | bigint(20)   |
| customer_id   | varchar(50)  |
| device_type   | varchar(32)  |
| device_token  | varchar(191) |

DDL:

    CREATE TABLE customer_device (
        id BIGINT PRIMARY KEY NOT NULL AUTO_INCREMENT,
        customer_id VARCHAR(50) NOT NULL,
        device_type VARCHAR(32) NOT NULL,
        device_token VARCHAR(191) NOT NULL
    );

## Run 

Make sure you have a Java JDK 8 (1.8) available:

    $ java -version
    java version "1.8.0_05"
    Java(TM) SE Runtime Environment (build 1.8.0_05-b13)
    Java HotSpot(TM) 64-Bit Server VM (build 25.5-b02, mixed mode)

The app can be started with the command:

    $ java -Dspring.config.location=/path/to/ipns-proxy.properties -jar ipns-proxy-ref-impl.jar
    
where ipns-proxy.properties contains the [configuration](#config).

## Usage

### Monitoring

Service status can be checked with a GET request:

    $ curl --user instaply:ChangeMe! -X GET http://localhost:8080/notifications
    {"timestamp":"2015-05-20T07:26:06.606Z"}

### Send a push notification

A push notification can be sent with a POST request:

    $ curl -X POST --user instaply:ChangeMe! --header "Content-Type:application/json" --header "ipns-version:1" -d @body.json http://localhost:8080/notifications

where body.json is a file containing the request body:

    {
      "type": "message",
      "unreadMessagesCount": 123,
      "date": "2015-05-01T14:26:39.546Z",
      "customer": {"id": "dng-1"},
      "message": {
        "from": {
          "user": {
            "firstName": "Fabien",
            "lastName": "Degaugue",
            "picture": "https://api.instaply.com/users/5/icon"
          },
          "organization": {
            "name": "Instaply",
            "picture": "https://api.instaply.com/files/de3d4c6b-0be4-4bb1-b534-b42272cc3998"
          }
        },
        "text": "hello sdk",
        "attachment": {
          "name": "instaply-logo.png",
          "contentType": "image/png",
          "thumbnails": {"_50x50": "https://api.instaply.com/files/de3d4c6b-0be4-4bb1-b534-b42272cc3998?width=50&height=50"}
        }
      }
    }
