package com.instaply.sdk.proxy;

import com.fasterxml.jackson.datatype.jdk8.Jdk8Module;
import com.fasterxml.jackson.datatype.jsr310.JSR310Module;

import com.notnoop.apns.APNS;
import com.notnoop.apns.ApnsService;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

@Configuration
public class WebConfiguration {
  /**
   * Datatype module to make Jackson recognize Java 8 types.
   * Note:
   * <ul>
   *   <li>New Date types are supported by JSR-310 module</li>
   *   <li>Support for parameter names is via Parameter Names module</li>
   * </ul>
   *
   * @see <a href="https://github.com/FasterXML/jackson-datatype-jdk8">https://github.com/FasterXML/jackson-datatype-jdk8</a>
   */
  @Bean
  public Jdk8Module jdk8Module() {
    return new Jdk8Module();
  }

  /**
   * Datatype module to make Jackson recognize Java 8 Date & Time API data types (JSR-310).
   * @see <a href="https://github.com/FasterXML/jackson-datatype-jsr310">https://github.com/FasterXML/jackson-datatype-jsr310</a>
   */
  @Bean
  public JSR310Module jsr310Module() {
    return new JSR310Module();
  }

  @Bean
  public ApnsService apnsService(@Value("${apns.certificate:apns-dev.p12}") String certificate,
                                 @Value("${apns.password:instaply}") String password,
                                 @Value("${apns.isProduction:false}") boolean isProduction) {
    return APNS.newService()
        .withCert(certificate, password)
        .withAppleDestination(isProduction)
        .build();
  }
}
