package com.instaply.sdk.proxy.dto;

import static com.fasterxml.jackson.annotation.JsonInclude.Include.NON_EMPTY;

import com.fasterxml.jackson.annotation.JsonInclude;

import java.util.Collections;
import java.util.List;
import java.util.UUID;

@JsonInclude(NON_EMPTY)
public class ExceptionDto {
  private final UUID uuid;
  private final String code;
  private final String message;
  private final List<ErrorDto> errors;

  public ExceptionDto(String code, String message) {
    this(code, message, Collections.<ErrorDto> emptyList());
  }

  public ExceptionDto(String code, String message, List<ErrorDto> errors) {
    this.uuid = UUID.randomUUID();
    this.code = code;
    this.message = message;
    this.errors = errors;
  }

  public UUID getUuid() {
    return uuid;
  }

  public String getCode() {
    return code;
  }

  public String getMessage() {
    return message;
  }

  public List<ErrorDto> getErrors() {
    return errors;
  }

  @Override
  public String toString() {
    return "ExceptionDto{" +
        "uuid=" + uuid +
        ", code='" + code + '\'' +
        ", message='" + message + '\'' +
        ", errors=" + errors +
        '}';
  }

  public static class ErrorDto  {
    final String field;
    final String message;

    public ErrorDto(String field, String message) {
      this.field = field;
      this.message = message;
    }

    public String getField() {
      return field;
    }

    public String getMessage() {
      return message;
    }

    @Override
    public String toString() {
      return "ErrorDto{" +
          "field='" + field + '\'' +
          ", message='" + message + '\'' +
          '}';
    }
  }
}
