package com.instaply.sdk.proxy.dto;

import com.instaply.sdk.proxy.validator.NotAllNull;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;

import org.hibernate.validator.constraints.NotBlank;
import org.hibernate.validator.constraints.URL;

@NotAllNull(value = {"text", "attachment"}, message = "text and attachment cannot be both null")
public class MessageDto {
  @NotNull
  @Valid
  private FromDto from;
  private String text;
  @Valid
  private AttachmentDto attachment;

  public static class FromDto {
    @Valid
    private OrganizationDto organization;
    @Valid
    private UserDto user;

    public OrganizationDto getOrganization() {
      return organization;
    }

    public UserDto getUser() {
      return user;
    }

    @Override
    public String toString() {
      return "FromDto{" +
          "organization=" + organization +
          ", user=" + user +
          '}';
    }
  }

  public static class AttachmentDto {
    @NotBlank
    private String name;
    @NotBlank
    private String contentType;
    @Valid
    private ThumbnailsDto thumbnails;

    public static class ThumbnailsDto {
      @NotBlank
      @URL
      private String _50x50;

      public String get_50x50() {
        return _50x50;
      }
    }

    public String getName() {
      return name;
    }

    public String getContentType() {
      return contentType;
    }

    public ThumbnailsDto getThumbnails() {
      return thumbnails;
    }

    @Override
    public String toString() {
      return "AttachmentDto{" +
          "name='" + name + '\'' +
          ", contentType='" + contentType + '\'' +
          '}';
    }
  }

  public FromDto getFrom() {
    return from;
  }

  public String getText() {
    return text;
  }

  public AttachmentDto getAttachment() {
    return attachment;
  }

  @Override
  public String toString() {
    return "MessageDto{" +
        "from=" + from +
        ", text='" + text + '\'' +
        ", attachment=" + attachment +
        '}';
  }
}
