package com.instaply.sdk.proxy.service;

import com.instaply.sdk.proxy.dto.MessageDto;
import com.instaply.sdk.proxy.dto.NotificationDto;
import com.instaply.sdk.proxy.dto.UserDto;

import com.notnoop.apns.APNS;

import javax.validation.constraints.NotNull;

import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

@Component(value = "messagePayloadFactory")
public class MessagePayloadFactory implements PayloadFactory {

  @Override
  public String apnsPayload(@NotNull NotificationDto notification) {
    MessageDto message = notification.getMessage();
    UserDto user = message.getFrom().getUser();

    String alertBody;
    String from = (user == null) ? "" : " from " + user.getFullName();
    if (!StringUtils.isEmpty(message.getText())) {
      alertBody = String.format("New message received%s: %s", from, message.getText());
    } else {
      MessageDto.AttachmentDto attachment = message.getAttachment();
      alertBody = String.format("New file received%s: %s", from, attachment.getName());
    }

    return APNS.newPayload()
        .badge(notification.getUnreadMessagesCount())
        .customField("type", "instaply_message")
        .alertBody(alertBody)
        .build();
  }
}
