package com.instaply.sdk.proxy.service;

import static java.util.stream.Collectors.toList;

import com.instaply.sdk.proxy.dto.NotificationDto;
import com.instaply.sdk.proxy.dto.NotificationType;
import com.instaply.sdk.proxy.entity.Device;
import com.instaply.sdk.proxy.repository.DeviceRepository;

import com.notnoop.apns.ApnsService;
import com.notnoop.apns.EnhancedApnsNotification;

import java.time.Instant;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;


@Service
public class PushNotificationService {

  private static final Logger logger = LoggerFactory.getLogger(PushNotificationService.class);
  private static final String APNS_TYPE = "apns";

  @Autowired
  private ApnsService apnsService;

  @Autowired
  private MessagePayloadFactory messagePayloadFactory;

  @Autowired
  private DeviceRepository deviceRepository;

  public void sendPushNotification(NotificationDto notification) {
    List<Device> devices = deviceRepository.findAllByCustomerId(notification.getCustomer().getId());
    if (devices.isEmpty()) {
      logger.info("No device found for customer {}, not sending any push notification", notification.getCustomer().getId());
      return;
    }

    // Send iOS push notification
    List<Device> iosDevices = devices.stream()
        .filter(device -> APNS_TYPE.equals(device.getType()))
        .collect(toList());

    if (notification.getType() == NotificationType.message && !iosDevices.isEmpty()) {
      sendApnsNotification(iosDevices, messagePayloadFactory.apnsPayload(notification));
    }
  }

  private void sendApnsNotification(List<Device> devices, String payload) {
    devices.parallelStream().forEach(device -> {
      int notificationId = EnhancedApnsNotification.INCREMENT_ID();

      logger.debug("Sending enhanced APNS notification [notificationId: {}, token: {}, payload: {}]",
          notificationId, device.getToken(), payload);

      apnsService.push(new EnhancedApnsNotification(
          notificationId,
          (int) Instant.now().plusSeconds(3600).getEpochSecond(),
          device.getToken(),
          payload));
    });
  }
}
