package com.instaply.sdk.proxy

import com.instaply.sdk.proxy.entity.Device
import com.instaply.sdk.proxy.repository.DeviceCrudRepository

import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.SpringApplicationContextLoader
import org.springframework.test.context.ContextConfiguration
import org.springframework.test.context.web.WebAppConfiguration
import org.springframework.test.web.servlet.MockMvc
import org.springframework.transaction.annotation.Transactional

import spock.lang.Specification

@WebAppConfiguration
@ContextConfiguration(classes = TestConfiguration.class, loader = SpringApplicationContextLoader.class)
@Transactional
class IntegrationSpec extends Specification {

  @Autowired
  MockMvc mockMvc

  @Autowired
  DeviceCrudRepository deviceCrudRepository

  def setup() {
    assert mockMvc != null
  }

  def putAt(Map map, String path, def value) {
    def token = path.tokenize('.')
    def key = token.pop()
    token.inject(map) { v, k -> v?."$k" }?.put(key, value)
  }

  def removeAt(Map map, String path) {
    if (!path) {
      map.clear()
    } else {
      def token = path.tokenize('.')
      def key = token.pop()
      token.inject(map) { v, k -> v?."$k" }?.remove(key)
    }
  }

  String randomToken() {
    return UUID.randomUUID().toString().replaceAll('-', '')
  }

  Device createAndSaveDevice(Long id, String customerId, String token, String type) {
    def device = new Device([id: id, customerId: customerId, token: token, type: type])
    deviceCrudRepository.save(device)
    return device
  }

  /**
   * Wrapping method to allow verifying Mockito mocks inside a Spock specification.
   *
   * Usage:
   * <pre>{@code
   * mockito ( verify ( someMock , times ( 1 ) ).someMethod(eq(param1)))
   *}</pre>
   *
   * <p>This method is needed to be able to verify Mockito mocks in the `then` block, because
   * it may only contain assertions in the form of boolean expressions. A Mockito verification
   * expression doesn’t fit this contract, as it will return a falsy value (null, false, 0)
   * when it passes, which is interpreted as a failed assertion by Spock.</p>
   *
   * @see <a href="http://stackoverflow.com/questions/16678126/simple-mockito-verify-works-in-junit-but-not-spock">
   *   http://stackoverflow.com/questions/16678126/simple-mockito-verify-works-in-junit-but-not-spock</a>
   *
   * @param verifyExpression a Mockito.verify() expression.
   * @return true.
   */
  boolean mockito(def expression) {
    return true
  }
}
